#include <complex>
#include <utility>
#include <vector>

#include "clutterCancellation.h"

using std::complex;
using std::pair;
using std::vector;

typedef vector<complex<double>> array;
typedef vector<array> mat;

/*
 * Description:
 * This function assumes that the surveillance and reference channels are digitalized and downloaded together
 * in the digital data acquisition stage, however the reference signal is received on a dedicated receiver and
 * antenna channel. At the output of the function the isolated reference signal and the rearranged matrix of
 * the surveillance channels will appear.
 * 
 * Parameters:
 * iq_samples_matrix:   This matrix contains the signal array download from
 *                      the receiver hardware, which contains all the antenna channels. M is the count of the
 *                      surveillance channels, while N is the number of samples in a channel.
 * ref_position:        Index of the reference channel in the iq_matrix.
 */
pair<array, mat> isolate_channels( 
    const mat& iq_samples_matrix,
    int ref_position);


/*
 * Description:  Filters the surveillance channel in the time and Doppler domain.
 * Uses the Extensive Cancellation Algorithm which is the extension of the standard time domain Wiener filter
 * in which the subspace of the reference signal is extended into the Doppler domain.
 * 
 * Parameters:
 * ref_ch : Complex signal samples of the reference channel
 * surv_ch : Contains the complex signal samples of the surveillance channel
 * K : Time domain filter dimension
 * D : Maximum Doppler extension measured in Doppler bins
 */
array time_domain_filter_surveillance(array& ref_ch, array& surv_ch, int K/*, int D*/);