#ifndef DETECTOR_H
#define DETECTOR_H

#include <algorithm>
#include <cmath>
#include <complex>
#include <string>
#include <valarray>
#include <vector>

#include "fft.h"
#include "numpy_helper.h"

using std::complex;
using std::cos;
using std::string;
using std::valarray;
using std::vector;

typedef vector<complex<double>> array;
typedef vector<array> mat;  


/*
 * Description:
 * Applies the specified window function on the surveillance channel. Using the appropriate window function
 * the energy leakage of the reflected signal component in the Doppler domain can be avoided.
 * 
 * Paramters:
 * surv_ch: Surveillance channel
 * window_function: (string) Name of the applied window function
 */
array windowing(const array& surv_ch, const string& window_function);


/*
 * Description: Cross-Correlation detection in the time domain
 * 
 * Parameters:
 * 
 * ref_ch: Reference channel
 * surv_ch: Surveillance channel
 * fs: Sampling frequency in Hz
 * fD_max: Maximum Doppler frequency
 * fD_s: Doppler frequency step size
 * r_max: Maximum range
 */
mat cc_detector_td(
    const array& ref_ch,
    const array& surv_ch,
    int fs,
    int fD_max,
    int fD_s,
    int r_max);

mat cc_detector_ons(
    array& ref_ch,
    array& surv_ch,
    int fs,
    int fD_max,
    int r_max);

#endif
