#ifndef FFT_H
#define FFT_H

#include <fftw3.h>

#include <cassert>
#include <complex>
#include <iostream>
#include <valarray>
#include <vector>

using std::cerr;
using std::complex;
using std::polar;
using std::valarray;
using std::vector;

typedef vector<complex<double>> array;


// To allow for input of vector as well
inline void fft(array& x, int sign=FFTW_FORWARD) {
	int N = static_cast<int>(x.size());

	// Use fftw
	fftw_complex *in, *out;
	fftw_plan p;
	in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * N);
    out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * N);
	int idx = 0;
	for (complex<double>& c : x) {
		in[idx][0] = c.real();
		in[idx++][1] = c.imag();
	}

	p = fftw_plan_dft_1d(N, in, out, sign, FFTW_ESTIMATE);
	fftw_execute(p);
	for (int i = 0; i < N; ++i) {
		if (sign == FFTW_BACKWARD) {
			x[i] = complex<double>(out[i][0], out[i][1]) / double(N);
		}
		else {
			x[i] = complex<double>(out[i][0], out[i][1]);
		}
	}
	fftw_destroy_plan(p);
	fftw_free(in);
	fftw_free(out);
}

// // inverse fft (in-place)
// inline void ifft(valarray<complex<double>>& x)
// {
//     fft(x, FFTW_BACKWARD);
// }

// To allow for input of vector as well
inline void ifft(array& x) {
	fft(x, FFTW_BACKWARD);
}

inline void fft(valarray<complex<double>>& x) {
	int N = static_cast<int>(x.size());
	array temp(x.size());
	for (int i = 0; i < N; ++i) {
		temp[i] = x[i];
	}

	fft(temp);

	for (int i = 0; i < N; ++i) {
		x[i] = temp[i];
	}
}

inline void ifft(valarray<complex<double>>& x) {
	int N = static_cast<int>(x.size());
	array temp(x.size());
	for (int i = 0; i < N; ++i) {
		temp[i] = x[i];
	}

	ifft(temp);

	for (int i = 0; i < N; ++i) {
		x[i] = temp[i];
	}
}

#endif
