#include "numpy_helper.h"

#include <cmath>
#include <complex>
#include <vector>

using std::abs;
using std::complex;
using std::log10;
using std::vector;

typedef vector<int> int_array;
typedef vector<double> double_array;
typedef vector<double_array> double_mat;
typedef vector<complex<double>> array;
typedef vector<array> mat;
typedef vector<mat> tens;

/*
 * Description:
 * Extracts the "Delta" detection performance metric from the given range-Doppler map
 * with specifing the true target coordinates. For the estimation of the variance of the 
 * noise, the utilized traning cells are extracted from the inmediate vicinity of the target 
 * correlation peak.
 * This detection metric is inspired by the CA-CFAR method.
 * 
 * Parameters:
 * rd_matrix: Range-Doppler map on which the metric extraction should be performed
 * target_rd: True target cell indices [range cell index, doppler cell index]
 * win      : Parameters of the noise power estimation window 
 *            [Est. window length, Est. window width, Guard window length, Guard window width]
 */
double extract_delta(mat& rd_matrix, int_array target_rd, int_array& win);